@@ -1,7 +1,7 @@
 use warnings;
 use strict;
 
-package TextFileParser 0.1821902;
+package TextFileParser 0.1821905;
 
 # ABSTRACT: an extensible Perl class to parse any text file by specifying grammar in derived classes.
 
@@ -84,6 +84,7 @@ sub __open_file {
 sub __read_file_handle {
     my ($self) = @_;
     my $fh = $self->{__filehandle};
+    $self->lines_parsed(0);
     while (<$fh>) {
         $self->lines_parsed( $self->lines_parsed + 1 );
         $self->__try_to_parse($_);
@@ -110,7 +111,7 @@ sub __try_to_parse {
 sub save_record {
     my $self = shift;
     return if not @_;
-    $self->{__records} = [] if not exists $self->{__records};
+    $self->{__records} = [] if not defined $self->{__records};
     push @{ $self->{__records} }, shift;
 }
 
@@ -141,6 +142,13 @@ sub last_record {
     return $record[$#record];
 }
 
+
+sub pop_record {
+    my $self = shift;
+    return undef if not exists $self->{__records};
+    pop @{ $self->{__records} };
+}
+
 1;
 
 __END__
@@ -155,7 +163,7 @@ TextFileParser - an extensible Perl class to parse any text file by specifying g
 
 =head1 VERSION
 
-version 0.1821902
+version 0.1821905
 
 =head1 SYNOPSIS
 
@@ -173,35 +181,24 @@ Here's another parser which is derived from C<TextFileParser> as the base class.
 
     use strict;
     use warnings;
-    package WordCounter;
+    package CSVParser;
     use parent 'TextFileParser';
-    use Exception::Class (
-        'SomeException' => {
-            description => 'Some description',
-            alias => 'throw_some_error'
-        }
-    );
 
     sub save_record {
         my ($self, $line) = @_;
-        my (@words) = split /\s+/, $line;
-        throw_some_error 
-            error => $self->filename . ': expected at least one character: ' . $self->lines_parsed
-                if not @words;
-        $self->SUPER::save_record(\@words);
+        chomp $line;
+        my (@fields) = split /,/, $line;
+        $self->SUPER::save_record(\@fields);
     }
 
     package main;
-    use Data::Dumper qw(Dumper);
-    use Try::Tiny;
 
-    try {
-        my $a_parser = new WordCounter;
-        $a_parser->read(shift @ARGV);
-        print Dumper($a_parser->get_records);
-    } catch {
-        print STDERR $_, "\n";
-    };
+    my $a_parser = new CSVParser;
+    $a_parser->read(shift @ARGV);
+    foreach my $rec ($a_parser->get_records) {
+        print $_, "\t" for (@{$rec});
+        print "\n";
+    }
 
 =head1 METHODS
 
@@ -264,20 +261,21 @@ Here's an example of a parser that reads multi-line records: if a line starts wi
 
     use strict;
     use warnings;
-    package MultilineWordCounter;
+    package MultilineParser;
     use parent 'TextFileParser';
 
     sub save_record {
         my ($self, $line) = @_;
-        my (@words) = split /\s+/, $line;
-        my $method = ($#words >= 0 and $words[0] eq '+') ? '__append_last_record' : 'SUPER::save_record';
-        $self->$method(\@words);
+        chomp $last_rec;
+        return $self->SUPER::save_record($line) if $line !~ /^[+]\s+/;
+        $line =~ s/^[+]\s+//g;
+        $self->SUPER::save_record( $self->pop_record . $line );
     }
 
     sub __append_last_record {
-        my ($self, $plus, $words) = @_;
-        my $last_rec = $self->last_record;
-        push @{last_rec}, @{$words};
+        my ($self, $line) = @_;
+        my $last_rec = $self->pop_record;
+        $self->SUPER::save_record($last_rec . $line);
     }
 
 =head2 get_records
@@ -290,7 +288,17 @@ Takes no arguments and returns the reference to the array containing all the rec
 
 =head2 last_record
 
-Takes no arguments and returns the last saved record.
+Takes no arguments and returns the last saved record. Leaves the saved records untouched.
+
+    my $last_rec = $pars->last_record;
+
+=head2 pop_record
+
+Takes no arguments and returns the last saved record. In the process the last element from the saved records is lost. To ensure data is not lost, make sure you call C<SUPER::save_record> or simply C<save_record> after a call to C<pop_record>.
+
+    my $last_rec = $pars->pop_record;
+    $uc_last = uc $last_rec;
+    $pars->save_record($uc_last);
 
 =head1 AUTHOR
 
